#include "opener_server.h"
#include "opener_resource.h"
#include "resource_server.h"

/* Support for providing access to files. */

OpenerResource::OpenerResource(Accessing *accessing, Paging *paging, Pages *pages)
: _accessing(accessing), _paging(paging), _pages(pages)
{
}

int OpenerResource::expected_items()
{
    return Opener_expected_items;
}

ipc_server_handler_type OpenerResource::handler()
{
    return (ipc_server_handler_type) handle_Opener;
}



/* Return a pager object for the given path and flags. */

FilePager *OpenerResource::open(const char *path, flags_t flags)
{
    fileid_t fileid = get_fileid(path);

    if (fileid == FILEID_INVALID)
        return NULL;

    Accessor *accessor = get_accessor(fileid);
    PageMapper *mapper = get_mapper(accessor);
    return new FilePager(fileid, mapper, flags);
}

/* Obtain an accessor for the 'fileid' or register a new one in the accessing
   object. */

Accessor *OpenerResource::get_accessor(fileid_t fileid)
{
    Accessor *accessor;

    if (_accessing->reserve(fileid))
        accessor = _accessing->get(fileid);
    else
    {
        accessor = make_accessor(fileid);
        _accessing->set(fileid, accessor);
    }

    return accessor;
}

/* Obtain a page mapper for the 'accessor' or register a new one in the
   paging object. */

PageMapper *OpenerResource::get_mapper(Accessor *accessor)
{
    PageMapper *mapper;

    if (_paging->reserve(accessor))
        mapper = _paging->get(accessor);
    else
    {
        mapper = new PageMapper(accessor, _pages);
        _paging->set(accessor, mapper);
    }

    return mapper;
}



/* Opener interface methods. */

long OpenerResource::context(l4_cap_idx_t *context)
{
    OpenerContextResource *resource = new OpenerContextResource(this);

    /* Complete the initialisation and start a server in a new thread. */

    ResourceServer server(resource);
    long err = server.start_thread();

    /* Return the server capability to the caller. */

    *context = server.config()->server;
    return err;
}

// vim: tabstop=4 expandtab shiftwidth=4
